@@ -1,4 +1,8 @@
 #!/usr/bin/env ruby
+# frozen_string_literal: true
+
+# Released under the MIT License.
+# Copyright, 2021-2022, by Samuel Williams.
 
 require 'async'
 
@@ -14,7 +18,7 @@ end
 
 Async do
 	progress = Console.logger.progress("Pool Usage", 10*10)
-	pool = Async::Pool::Controller.new(MyResource)
+	pool = Async::Pool::Controller.new(MyResource, concurrency: 10)
 	
 	10.times do
 		Async do
