@@ -1,22 +1,8 @@
-# Copyright, 2017, by Samuel G. D. Williams. <http://www.codeotaku.com>
-# 
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-# 
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-# 
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-# THE SOFTWARE.
+# frozen_string_literal: true
+
+# Released under the MIT License.
+# Copyright, 2019-2022, by Samuel Williams.
+# Copyright, 2020, by Simon Perepelitsa.
 
 require 'console/logger'
 
@@ -31,7 +17,7 @@ module Async
 				self.new(block, **options)
 			end
 			
-			def initialize(constructor, limit: nil)
+			def initialize(constructor, limit: nil, concurrency: nil)
 				# All available resources:
 				@resources = {}
 				
@@ -44,7 +30,15 @@ module Async
 				@limit = limit
 				
 				@constructor = constructor
-				@guard = Async::Semaphore.new(1)
+				
+				# Set the concurrency to be the same as the limit for maximum performance:
+				if limit
+					concurrency ||= limit
+				else
+					concurrency ||= 1
+				end
+				
+				@guard = Async::Semaphore.new(concurrency)
 				
 				@gardener = nil
 			end
@@ -98,21 +92,28 @@ module Async
 			
 			# Make the resource resources and let waiting tasks know that there is something resources.
 			def release(resource)
-				reused = false
+				processed = false
 				
 				# A resource that is not good should also not be reusable.
 				if resource.reusable?
-					reused = reuse(resource)
+					processed = reuse(resource)
 				end
 			ensure
-				retire(resource) unless reused
+				retire(resource) unless processed
 			end
 			
 			def close
 				@available.clear
 				
-				@resources.each_key(&:close)
-				@resources.clear
+				while pair = @resources.shift
+					resource, usage = pair
+					
+					if usage > 0
+						Console.logger.warn(self, resource: resource, usage: usage) {"Closing resource while still in use!"}
+					end
+					
+					resource.close
+				end
 				
 				@gardener&.stop
 			end
@@ -131,12 +132,14 @@ module Async
 			def prune(retain = 0)
 				unused = []
 				
+				# This code must not context switch:
 				@resources.each do |resource, usage|
 					if usage.zero?
 						unused << resource
 					end
 				end
 				
+				# It's okay for this to context switch:
 				unused.each do |resource|
 					if block_given?
 						yield resource
@@ -166,6 +169,8 @@ module Async
 				resource.close
 				
 				@notification.signal
+				
+				return true
 			end
 			
 			protected
@@ -201,13 +206,6 @@ module Async
 				@resources.count{|resource, usage| usage == 0}
 			end
 			
-			# @returns [Boolean] Whether the number of available resources is excessive and we should retire some.
-			def overflowing?
-				if @resources.any?
-					(self.free.to_f / @resources.size) > 0.5
-				end
-			end
-			
 			def reuse(resource)
 				Console.logger.debug(self) {"Reuse #{resource}"}
 				usage = @resources[resource]
@@ -216,13 +214,6 @@ module Async
 					raise "Trying to reuse unacquired resource: #{resource}!"
 				end
 				
-				# We retire resources when adding to the @available list would overflow our pool:
-				if usage == 1
-					if overflowing?
-						return retire(resource)
-					end
-				end
-				
 				# If the resource was fully utilized, it now becomes available:
 				if usage == resource.concurrency
 					@available.push(resource)
